/* Definitions for DISLITE. */

/* INIBLOCK: room for the first few bytes of the file, to locate the
"PKLITE" header */
#define INIBLOCK 256	/* Ver 1.15 has larger exe header. */
/* MAXFILELEN: the maximum file+path length in characters */
#define MAXFILELEN 100

/* {MAX|MIN}ALLOCSIZE: constants for allocating some storage */
#define MAXALLOCSIZE 60000U
#define MINALLOCSIZE 2048

/* ENDOFSEGLIST indicates the end of the segments listed in fixupseg */
#define ENDOFSEGLIST 0xEFFFU
/* COMPAREOFFSET is the amount added to compare relative segments */
#define COMPAREOFFSET 0x1000U

/* SAFETYGUARD: extra space accounted when loading EXEs, just in case... */
#define SAFETYGUARD 512

/* PLAYSPACESIZE: space on the heap to leave free for various operations */
#define PLAYSPACESIZE 4096

/* ZEROLEN: lenght of a piece of memory filled with zeros */
#define ZEROLEN 256

/* ASM_*: opcodes for ASM instructions */
#define ASM_BREAKPOINT 0xCC
#define ASM_INT 0xCD
#define ASM_NOP 0x90
#define ASM_JMPSHORT 0xEB
#define ASM_JMPNEAR 0xE9
#define ASM_JMPFAR 0xEA
#define ASM_RET 0xC3
#define ASM_RETF 0xCB

/* EXESIG?: Signature of EXE files */
#define EXESIG1 0x4D5A
#define EXESIG2 0x5A4D

/* SINGLESTEPFLAG: The bit in the FLAGS word that triggers single steps */
#define SINGLESTEPFLAG 0x0100

/* SINGLESTEPINT,BREAKPOINTINT: # of a single step or breakpoint int */
#define SINGLESTEPINT 1
#define BREAKPOINTINT 3

/* BIOSINT,DISKINT,DOSINT: # of these interrupts */
#define BIOSINT 0x10
#define DISKINT 0x13
#define DOSINT 0x21

/* DOSINTVECT: the actual location in memory where the INT 21 vector is stored */
#define DOSINTVECT ((void interrupt (*far*)(void)) MK_FP(0, 0x84))

/* GETSWITCHAR,LOADNOTEXEC: some INT 21 calls */
#define GETSWITCHAR 0x3700
#define LOADNOTEXEC 0x4B01

/* FIXUPINT: the interrupt that comes instead of a fixup instruction */
#define FIXUPINT 0xF1

/* MAXCHUNK,MINCHUNK: memory used when copying overlays */
#define MAXCHUNK 0x8000U
#define MINCHUNK 0x0200

/* FLAGS_LARGE,FLAGS_EXTRA: the bits in the flags nybble from the PKLITE header */
#define FLAGS_LARGE 0x2
#define FLAGS_EXTRA 0x1

/* FLAGS_NOPK: set when the "PKLITE" string isn't found, no version info */
#define FLAGS_NOPK 0x80

/* Format of an .EXE file header */
struct Exe_Header {
   unsigned signature;
   unsigned bytesinlastpage;
   unsigned numberofpages;
   unsigned relocentries;
   unsigned headersize;	/* In paragraphs */
   unsigned minalloc;
   unsigned maxalloc;
   unsigned relSS;
   unsigned SP;
   unsigned checksum;
   unsigned IP;
   unsigned relCS;
   unsigned reloctablep;
   unsigned overlaynum;
};

/* The structure that is passed to the LOADNOTEXEC call */
struct Load_Exe {
   unsigned envp;
   void far* commandtail;
   void far* firstFCB;
   void far* secondFCB;
   void far* SSSP;
   void far* CSIP;
};

/* Template PSP (only defines the exit address) */
struct Psp {
   char not_needed[10];
   void (interrupt far* exitaddr)(void);
};

/* The options packed structure. Contains 1 bit for every toggle option on
 the DISLITE command line */
struct Options {
   int makebackup:1;
   int noheaderinf:1;
   int overwrite:1;
   int pageheader:1;
   int nooverlay:1;
   int silent:1;
   int nosignature:1;
   int settime:1;
   int dofixfixups:1;
   int listonly:1;
};

/* The state of the currently executing program to be disLITEd */
struct Exe_State {
   void far* rSSSP;
   void far* rCSIP;
   struct Psp far* psp;
   unsigned rAX, rBX, rCX, rDX, rSI, rDI, rBP, rDS, rES, rFLAGS;
};

/* Function templates */
static char* getnamepart(char* path);
static char far* xmatch(char far*, unsigned, char*);
static char far* multixmatch(char far*, unsigned, char**);
static void scanfile(void);
static void parseargs(int, char**);
static int isoption(char*);
static long pages2long(struct Exe_Header*);
static void long2pages(long, struct Exe_Header*);
static void usage(void);
static void title(void);
static void allocbuf(void);
static void loadexe(void);
static void execuntil(char far*);
void interrupt fixup(void);
void interrupt unexit(void);
static void doubleexit(char*, int);
void interrupt normalexit(void);
static void addseg(unsigned);
static void preparenew(struct Exe_Header*, unsigned far*);
static void addpksign(struct Exe_Header* exeh);
static long teststoredheader(struct Exe_Header*, struct Exe_Header*);
static void fixfixups(int);
static void writeexe(struct Exe_Header*);

/* Function templates for the .ASM routines */
extern void interrupt resume(void);
extern void interrupt unbiosint(void);
extern void interrupt undiskint(void);
extern void interrupt undosint(void);
extern void terminate(void);
extern void switchto(void);
